<?php

declare(strict_types=1);
/**
 *  +----------------------------------------------------------------------
 *  | 陀螺匠 [ 赋能开发者，助力企业发展 ]
 *  +----------------------------------------------------------------------
 *  | Copyright (c) 2016~2023 https://www.tuoluojiang.com All rights reserved.
 *  +----------------------------------------------------------------------
 *  | Licensed 陀螺匠并不是自由软件，未经许可不能去掉陀螺匠相关版权
 *  +----------------------------------------------------------------------
 *  | Author: 陀螺匠 Team <admin@tuoluojiang.com>
 *  +----------------------------------------------------------------------
 *  | @author: lunatic<453850531@qq.com>
 *  +----------------------------------------------------------------------
 */
namespace App\Http\Service\File;

use App\Http\Dao\system\attach\FolderHistoryDao;
use App\Http\Dao\system\attach\FolderShareDao;
use crmeb\basic\BaseService;

/**
 * 文件分享表.
 * @mixin FolderShareDao
 */
class FolderHistoryService extends BaseService
{
    public function __construct(FolderHistoryDao $dao)
    {
        $this->dao = $dao;
    }

    public function folderHistory($folderId, $page, $limit)
    {
        $folder = app()->make(FileService::class)->get(['file_sn' => $folderId], [], ['user']);
        if (! $folder) {
            return [];
        }

        $lst = $this->search(['folder_id' => $folder->id])
            ->offset($page)->limit($limit)->with(['user'])->orderBy('version', 'DESC')->get()->toArray();

        $data = [];
        foreach ($lst as $item) {
            $data[] = [
                'id'           => $folder->file_sn,
                'size'         => (int) $item['file_size'],
                'name'         => $folder->name,
                'version'      => $item['version'],
                'uid'          => $folder->user->uid,
                'file_id'      => $folderId,
                'create_time'  => is_string($folder->created_at) ? strtotime($folder->created_at) : $folder->created_at->timestamp,
                'modify_time'  => strtotime($item['created_at']),
                'download_url' => get_download_url((string)$item['folder_id'], ['version' => $item['version'], 'folder' => 1]),
                'creator'      => [
                    'id'         => $folder->user->uid,
                    'name'       => $folder->user->account ?: $folder->user->real_name,
                    'avatar_url' => $folder->user->avatar,
                ],
                'modifier' => [
                    'id'         => $item['uid'],
                    'name'       => $item['user']['account'] ?: $item['user']['real_name'],
                    'avatar_url' => $item['user']['avatar'],
                ],
            ];
        }
        return $data;
    }

    /**
     * 最近更新信息.
     * @return mixed
     */
    public function lastHistory(int $folderId)
    {
        $info = $this->get(['folder_id' => $folderId], ['id', 'uid', 'created_at'], ['user' => fn ($q) => $q->select(['uid', 'real_name', 'avatar'])], 'created_at');
        return $info?->toArray();
    }
}
